package com.order.controller.business;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.hfkj.common.exception.ErrorCode;
import com.hfkj.common.exception.ErrorHelp;
import com.hfkj.common.exception.SysCode;
import com.hfkj.common.security.SessionObject;
import com.hfkj.common.security.UserCenter;
import com.hfkj.common.utils.OrderUtil;
import com.hfkj.common.utils.ResponseMsgUtil;
import com.hfkj.entity.BsOrderGoods;
import com.hfkj.entity.GoodsLogistics;
import com.hfkj.model.ResponseData;
import com.hfkj.model.SecUserSessionObject;
import com.hfkj.service.goods.BsOrderGoodsService;
import com.hfkj.service.order.BsOrderService;
import com.hfkj.sysenum.SecUserObjectTypeEnum;
import com.hfkj.sysenum.order.OrderGoodsStatusEnum;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

@Controller
@RequestMapping(value="/orderGoods")
@Api(value="实物订单管理")
public class BsOrderGoodsController {

    private static final Logger log = LoggerFactory.getLogger(BsOrderGoodsController.class);

    @Resource
    private BsOrderGoodsService bsOrderGoodsService;

    @Resource
    private UserCenter userCenter;

    @Resource
    private BsOrderService orderService;


    @RequestMapping(value="/getListGoods",method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "查询列表")
    public ResponseData getListGoods(@RequestParam(value = "logisticsNo" , required = false) String logisticsNo,
                                     @RequestParam(value = "goodsType" , required = false) Long goodsType,
                                     @RequestParam(value = "goodsBrand" , required = false) Long goodsBrand,
                                     @RequestParam(value = "orderNo" , required = false) String orderNo,
                                     @RequestParam(value = "childOrderNo" , required = false) String childOrderNo,
                                     @RequestParam(value = "userPhone" , required = false) String userPhone,
                                     @RequestParam(value = "goodsSpecsName" , required = false) String goodsSpecsName,
                                     @RequestParam(value = "goodsName" , required = false) String goodsName,
                                     @RequestParam(value = "payType" , required = false) Integer payType,
                                     @RequestParam(value = "status" , required = false) Integer status,
                                     @RequestParam(value = "createTimeS" , required = false) Long createTimeS,
                                     @RequestParam(value = "createTimeE" , required = false) Long createTimeE,
                                     @RequestParam(value = "pageNum" , required = true) Integer pageNum,
                                     @RequestParam(value = "pageSize" , required = true) Integer pageSize, HttpServletRequest request) {
        try {

            SessionObject sessionObject = userCenter.getSessionObject(request);
            SecUserSessionObject userModel = (SecUserSessionObject) sessionObject.getObject();

            Map<String , Object> map = new HashMap<>();

            if (userModel.getAccount().getObjectType().equals(SecUserObjectTypeEnum.type2.getCode())) {
                map.put("merId", userModel.getAccount().getId());
            }

            map.put("logisticsNo", logisticsNo);
            map.put("goodsType", goodsType);
            map.put("goodsBrand", goodsBrand);
            map.put("orderNo", orderNo);
            map.put("childOrderNo", childOrderNo);
            map.put("userPhone", userPhone);
            map.put("goodsSpecsName", goodsSpecsName);
            map.put("goodsName", goodsName);
            map.put("payType", payType);
            map.put("createTimeS", createTimeS);
            map.put("createTimeE", createTimeE);

            map.put("status", status);

            PageHelper.startPage(pageNum, pageSize);
            return ResponseMsgUtil.success(new PageInfo<>(bsOrderGoodsService.getList(map)));

        } catch (Exception e) {
            log.error("error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value="/updateLogisticsNo",method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "更新物流订单号")
    public ResponseData updateLogisticsNo(@RequestBody BsOrderGoods body
    ) {
        try {

            if (body == null
                    || body.getId() == null
                    || StringUtils.isBlank(body.getLogisticsNo())
            ) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.REQ_PARAMS_ERROR, "");
            }

            BsOrderGoods goodsOrder = bsOrderGoodsService.queryDetail(body.getId());

            if (goodsOrder == null) {
                throw ErrorHelp.genException(SysCode.System , ErrorCode.COMMON_ERROR , "信息错误！");
            }

            goodsOrder.setLogisticsNo(body.getLogisticsNo());
            goodsOrder.setUpdateTime(new Date());
            if (goodsOrder.getStatus() == OrderGoodsStatusEnum.status1.getCode()) {
                goodsOrder.setStatus(OrderGoodsStatusEnum.status2.getCode());
            }

            // 删除缓存
            orderService.cacheDelete(goodsOrder.getOrderNo());
            bsOrderGoodsService.cacheDelete(goodsOrder.getLogisticsNo());
            bsOrderGoodsService.update(goodsOrder);
            return ResponseMsgUtil.success("更新成功！");

        } catch (Exception e) {
            log.error("OrderExtController -> getGasOrderDetail() error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value="/getLogisticsMsg",method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "查询物流")
    public ResponseData getLogisticsMsg(@RequestParam(value = "id" , required = false) Long id) {
        try {
            Map<String , Object> map = new HashMap<>();

            BsOrderGoods orderGoods = bsOrderGoodsService.queryDetail(id);
            if (orderGoods == null || orderGoods.getLogisticsNo() == null) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "暂未发货！");
            }

            GoodsLogistics logistics = (GoodsLogistics) bsOrderGoodsService.getLogisticsMsg(orderGoods.getLogisticsNo());

            orderGoods.setLogisticsStatus(logistics.getLogisticsStatus());
            orderGoods.setLogisticsStatusDesc(logistics.getLogisticsStatusDesc());
            bsOrderGoodsService.update(orderGoods);

            map.put("logistics", logistics);
            JSONArray object = JSONArray.parseArray(logistics.getLogisticsTraceDetails());
            map.put("logisticsTraceDetails",  object);

            return ResponseMsgUtil.success(map);

        } catch (Exception e) {
            log.error("error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value="/confirmReceipt",method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "确认收货")
    public ResponseData confirmReceipt(@RequestParam(value = "id" , required = false) Long id) {
        try {
            bsOrderGoodsService.confirmReceipt(id);

            return ResponseMsgUtil.success("成功");

        } catch (Exception e) {
            log.error("error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }
}
